@@ -1,6 +1,6 @@
 // This defines the interface to the QsciLexerCPP class.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -115,9 +110,8 @@ public:
         CommentLineDoc = 15,
         InactiveCommentLineDoc = CommentLineDoc + 64,
 
-        //! A keyword defined in keyword set number 2.  The class must
-        //! be sub-classed and re-implement keywords() to make use of
-        //! this style.
+        //! A keyword defined in keyword set number 2.  The class must be
+        //! sub-classed and re-implement keywords() to make use of this style.
         KeywordSet2 = 16,
         InactiveKeywordSet2 = KeywordSet2 + 64,
 
@@ -129,15 +123,31 @@ public:
         CommentDocKeywordError = 18,
         InactiveCommentDocKeywordError = CommentDocKeywordError + 64,
 
-        //! A global class or typedef defined in keyword set number 5.
-        //! The class must be sub-classed and re-implement keywords()
-        //! to make use of this style.
+        //! A global class or typedef defined in keyword set number 5.  The
+        //! class must be sub-classed and re-implement keywords() to make use
+        //! of this style.
         GlobalClass = 19,
         InactiveGlobalClass = GlobalClass + 64,
 
         //! A C++ raw string.
         RawString = 20,
         InactiveRawString = RawString + 20,
+
+        //! A Vala triple-quoted verbatim string.
+        TripleQuotedVerbatimString = 21,
+        InactiveTripleQuotedVerbatimString = TripleQuotedVerbatimString + 21,
+
+        //! A Pike hash-quoted string.
+        HashQuotedString = 22,
+        InactiveHashQuotedString = HashQuotedString + 22,
+
+        //! A pre-processor stream comment.
+        PreProcessorComment = 23,
+        InactivePreProcessorComment = PreProcessorComment + 23,
+
+        //! A JavaDoc/Doxygen style pre-processor comment.
+        PreProcessorCommentLineDoc = 24,
+        InactivePreProcessorCommentLineDoc = PreProcessorCommentLineDoc + 64,
     };
 
     //! Construct a QsciLexerCPP with parent \a parent.  \a parent is typically
@@ -177,7 +187,7 @@ public:
     //! \internal Returns the style used for braces for brace matching.
     int braceStyle() const;
 
-    //! \internal Returns the string of characters that comprise a word.
+    //! Returns the string of characters that comprise a word.
     const char *wordCharacters() const;
 
     //! Returns the foreground colour of the text for style number \a style.
@@ -197,7 +207,11 @@ public:
     QColor defaultPaper(int style) const;
 
     //! Returns the set of keywords for the keyword set \a set recognised
-    //! by the lexer as a space separated string.
+    //! by the lexer as a space separated string.  Set 1 is normally used for
+    //! primary keywords and identifiers.  Set 2 is normally used for secondary
+    //! keywords and identifiers.  Set 3 is normally used for documentation
+    //! comment keywords.  Set 4 is normally used for global classes and
+    //! typedefs.
     const char *keywords(int set) const;
 
     //! Returns the descriptive name for style number \a style.  If the
@@ -246,6 +260,28 @@ public:
     //! \sa setDollarsAllowed()
     bool dollarsAllowed() const {return dollars;}
 
+    //! If \a enabled is true then triple quoted strings are highlighted.  The
+    //! default is false.
+    //!
+    //! \sa highlightTripleQuotedStrings()
+    void setHighlightTripleQuotedStrings(bool enabled);
+
+    //! Returns true if triple quoted strings should be highlighted.
+    //!
+    //! \sa setHighlightTripleQuotedStrings()
+    bool highlightTripleQuotedStrings() const {return highlight_triple;}
+
+    //! If \a enabled is true then hash quoted strings are highlighted.  The
+    //! default is false.
+    //!
+    //! \sa highlightHashQuotedStrings()
+    void setHighlightHashQuotedStrings(bool enabled);
+
+    //! Returns true if hash quoted strings should be highlighted.
+    //!
+    //! \sa setHighlightHashQuotedStrings()
+    bool highlightHashQuotedStrings() const {return highlight_hash;}
+
 public slots:
     //! If \a fold is true then "} else {" lines can be folded.  The
     //! default is false.
@@ -299,6 +335,8 @@ private:
     void setPreprocProp();
     void setStylePreprocProp();
     void setDollarsProp();
+    void setHighlightTripleProp();
+    void setHighlightHashProp();
 
     bool fold_atelse;
     bool fold_comments;
@@ -306,6 +344,8 @@ private:
     bool fold_preproc;
     bool style_preproc;
     bool dollars;
+    bool highlight_triple;
+    bool highlight_hash;
 
     bool nocase;
 
