@@ -1,6 +1,6 @@
 // This defines the interface to the QsciLexerCSS class.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -125,7 +120,10 @@ public:
         ExtendedPseudoElement = 21,
 
         //! A media rule.
-        MediaRule = 22
+        MediaRule = 22,
+
+        //! A variable.
+        Variable = 23,
     };
 
     //! Construct a QsciLexerCSS with parent \a parent.  \a parent is typically
@@ -152,7 +150,7 @@ public:
     //! auto-indentation.  The styles is returned via \a style.
     const char *blockStart(int *style = 0) const;
 
-    //! \internal Returns the string of characters that comprise a word.
+    //! Returns the string of characters that comprise a word.
     const char *wordCharacters() const;
 
     //! Returns the foreground colour of the text for style number \a style.
@@ -184,6 +182,39 @@ public:
     //! \sa setFoldCompact()
     bool foldCompact() const;
 
+    //! If \a enabled is true then support for HSS is enabled.  The default is
+    //! false.
+    //!
+    //! \sa HSSLanguage()
+    void setHSSLanguage(bool enabled);
+
+    //! Returns true if support for HSS is enabled.
+    //!
+    //! \sa setHSSLanguage()
+    bool HSSLanguage() const {return hss_language;}
+
+    //! If \a enabled is true then support for Less CSS is enabled.  The
+    //! default is false.
+    //!
+    //! \sa LessLanguage()
+    void setLessLanguage(bool enabled);
+
+    //! Returns true if support for Less CSS is enabled.
+    //!
+    //! \sa setLessLanguage()
+    bool LessLanguage() const {return less_language;}
+
+    //! If \a enabled is true then support for Sassy CSS is enabled.  The
+    //! default is false.
+    //!
+    //! \sa SCSSLanguage()
+    void setSCSSLanguage(bool enabled);
+
+    //! Returns true if support for Sassy CSS is enabled.
+    //!
+    //! \sa setSCSSLanguage()
+    bool SCSSLanguage() const {return scss_language;}
+
 public slots:
     //! If \a fold is true then multi-line comment blocks can be folded.
     //! The default is false.
@@ -213,9 +244,15 @@ protected:
 private:
     void setCommentProp();
     void setCompactProp();
+    void setHSSProp();
+    void setLessProp();
+    void setSCSSProp();
 
     bool fold_comments;
     bool fold_compact;
+    bool hss_language;
+    bool less_language;
+    bool scss_language;
 
     QsciLexerCSS(const QsciLexerCSS &);
     QsciLexerCSS &operator=(const QsciLexerCSS &);
