@@ -1,7 +1,7 @@
 // This module defines the "official" high-level API of the Qt port of
 // Scintilla.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -17,13 +17,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -44,12 +39,16 @@ extern "C++" {
 #include <QPointer>
 
 #include <Qsci/qsciglobal.h>
+#include <Qsci/qscicommand.h>
 #include <Qsci/qscidocument.h>
 #include <Qsci/qsciscintillabase.h>
 
 
 QT_BEGIN_NAMESPACE
+class QAction;
+class QImage;
 class QIODevice;
+class QMenu;
 class QPoint;
 QT_END_NAMESPACE
 
@@ -57,7 +56,7 @@ class QsciCommandSet;
 class QsciLexer;
 class QsciStyle;
 class QsciStyledText;
-class ListBoxQt;
+class QsciListBoxQt;
 
 
 //! \brief The QsciScintilla class implements a higher level, more Qt-like,
@@ -151,6 +150,15 @@ public:
         SloppyBraceMatch
     };
 
+    //! This enum defines the different call tip positions.
+    enum CallTipsPosition {
+        //! Call tips are placed below the text.
+        CallTipsBelowText,
+
+        //! Call tips are placed above the text.
+        CallTipsAboveText,
+    };
+
     //! This enum defines the different call tip styles.
     enum CallTipsStyle {
         //! Call tips are disabled.
@@ -193,10 +201,10 @@ public:
         //! A carriage return/line feed as used on Windows systems.
         EolWindows = SC_EOL_CRLF,
 
-        //! A line feed as used on Unix systems.
+        //! A line feed as used on Unix systems, including OS/X.
         EolUnix = SC_EOL_LF,
 
-        //! A carriage return as used on Mac systems.
+        //! A carriage return as used on Mac systems prior to OS/X.
         EolMac = SC_EOL_CR
     };
 
@@ -225,30 +233,66 @@ public:
 
     //! This enum defines the different indicator styles.
     enum IndicatorStyle {
-        // A single straight underline.
+        //! A single straight underline.
         PlainIndicator = INDIC_PLAIN,
 
-        // A squiggly underline.
+        //! A squiggly underline that requires 3 pixels of descender space.
         SquiggleIndicator = INDIC_SQUIGGLE,
 
-        // A line of small T shapes.
+        //! A line of small T shapes.
         TTIndicator = INDIC_TT,
 
-        // Diagonal hatching.
+        //! Diagonal hatching.
         DiagonalIndicator = INDIC_DIAGONAL,
 
-        // Strike out.
+        //! Strike out.
         StrikeIndicator = INDIC_STRIKE,
 
-        // An indicator with no visual appearence.
+        //! An indicator with no visual appearence.
         HiddenIndicator = INDIC_HIDDEN,
 
-        // A rectangle around the text.
+        //! A rectangle around the text.
         BoxIndicator = INDIC_BOX,
 
-        // A rectangle with rounded corners around the text with the interior
-        // more transparent than the border.
-        RoundBoxIndicator = INDIC_ROUNDBOX
+        //! A rectangle with rounded corners around the text with the interior
+        //! usually more transparent than the border.
+        RoundBoxIndicator = INDIC_ROUNDBOX,
+
+        //! A rectangle around the text with the interior usually more
+        //! transparent than the border.
+        StraightBoxIndicator = INDIC_STRAIGHTBOX,
+
+        //! A dashed underline.
+        DashesIndicator = INDIC_DASH,
+
+        //! A dotted underline.
+        DotsIndicator = INDIC_DOTS,
+
+        //! A squiggly underline that requires 2 pixels of descender space and
+        //! so will fit under smaller fonts.
+        SquiggleLowIndicator = INDIC_SQUIGGLELOW,
+
+        //! A dotted rectangle around the text with the interior usually more
+        //! transparent than the border.
+        DotBoxIndicator = INDIC_DOTBOX,
+
+        //! A version of SquiggleIndicator that uses a pixmap.  This is quicker
+        //! but may be of lower quality.
+        SquigglePixmapIndicator = INDIC_SQUIGGLEPIXMAP,
+
+        //! A thick underline typically used during Asian language input
+        //! composition.
+        ThickCompositionIndicator = INDIC_COMPOSITIONTHICK,
+    };
+
+    //! This enum defines the different margin options.
+    enum {
+        //! Reset all margin options.
+        MoNone = SC_MARGINOPTION_NONE,
+
+        //! If this is set then only the first sub-line of a wrapped line will
+        //! be selected when clicking on a margin.
+        MoSublineSelect = SC_MARGINOPTION_SUBLINESELECT
     };
 
     //! This enum defines the different margin types.
@@ -404,7 +448,10 @@ public:
         WrapFlagByText,
 
         //! A wrap flag is displayed by the border.
-        WrapFlagByBorder
+        WrapFlagByBorder,
+
+        //! A wrap flag is displayed in the line number margin.
+        WrapFlagInMargin
     };
 
     //! This enum defines the different line wrap indentation modes.
@@ -530,6 +577,11 @@ public:
     //! \sa setBraceMatching()
     BraceMatch braceMatching() const {return braceMode;}
 
+    //! Returns the current call tip position.
+    //!
+    //! \sa setCallTipsPosition()
+    CallTipsPosition callTipsPosition() const {return call_tips_position;}
+
     //! Returns the current call tip style.
     //!
     //! \sa setCallTipsStyle()
@@ -583,6 +635,11 @@ public:
     //! \sa eolMode(), setEolMode()
     void convertEols(EolMode mode);
 
+    //! Create the standard context menu which is shown when the user clicks
+    //! with the right mouse button.  It is called from contextMenuEvent().
+    //! The menu's ownership is transferred to the caller.
+    QMenu *createStandardContextMenu();
+
     //! Returns the attached document.
     //!
     //! \sa setDocument()
@@ -645,7 +702,7 @@ public:
     void fillIndicatorRange(int lineFrom, int indexFrom, int lineTo,
             int indexTo, int indicatorNumber);
 
-    //! Find the next occurrence of the string \a expr and return true if
+    //! Find the first occurrence of the string \a expr and return true if
     //! \a expr was found, otherwise returns false.  If \a expr is found it
     //! becomes the current selection.
     //!
@@ -670,14 +727,49 @@ public:
     //! If \a show is true (the default) then any text found is made visible
     //! (ie. it is unfolded).
     //!
-    //! \sa findNext(), replace()
+    //! If \a posix is true then a regular expression is treated in a more
+    //! POSIX compatible manner by interpreting bare ( and ) as tagged sections
+    //! rather than \( and \).
+    //!
+    //! \sa findFirstInSelection(), findNext(), replace()
     virtual bool findFirst(const QString &expr, bool re, bool cs, bool wo,
             bool wrap, bool forward = true, int line = -1, int index = -1,
-            bool show = true);
+            bool show = true, bool posix = false);
 
-    //! Find the next occurence of the string found using findFirst().
+    //! Find the first occurrence of the string \a expr in the current
+    //! selection and return true if \a expr was found, otherwise returns
+    //! false.  If \a expr is found it becomes the current selection.  The
+    //! original selection is restored when a subsequent call to findNext()
+    //! returns false.
     //!
-    //! \sa findFirst(), replace()
+    //! If \a re is true then \a expr is interpreted as a regular expression
+    //! rather than a simple string.
+    //!
+    //! If \a cs is true then the search is case sensitive.
+    //!
+    //! If \a wo is true then the search looks for whole word matches only,
+    //! otherwise it searches for any matching text.
+    //!
+    //! If \a forward is true (the default) then the search is forward from the
+    //! start to the end of the selection, otherwise it is backwards from the
+    //! end to the start of the selection.
+    //!
+    //! If \a show is true (the default) then any text found is made visible
+    //! (ie. it is unfolded).
+    //!
+    //! If \a posix is true then a regular expression is treated in a more
+    //! POSIX compatible manner by interpreting bare ( and ) as tagged sections
+    //! rather than \( and \).
+    //!
+    //! \sa findFirstInSelection(), findNext(), replace()
+    virtual bool findFirstInSelection(const QString &expr, bool re, bool cs,
+            bool wo, bool forward = true, bool show = true,
+            bool posix = false);
+
+    //! Find the next occurence of the string found using findFirst() or
+    //! findFirstInSelection().
+    //!
+    //! \sa findFirst(), findFirstInSelection(), replace()
     virtual bool findNext();
 
     //! Returns the number of the first visible line.
@@ -834,6 +926,11 @@ public:
     //! \sa setMarginMask(), QsciMarker, SCI_GETMARGINMASKN
     int marginMarkerMask(int margin) const;
 
+    //! Returns the margin options.  The default is MoNone.
+    //!
+    //! \sa setMarginOptions(), MoNone, MoSublineSelect.
+    int marginOptions() const;
+
     //! Returns true if margin \a margin is sensitive to mouse clicks.
     //!
     //! \sa setMarginSensitivity(), marginClicked(), SCI_GETMARGINTYPEN
@@ -883,6 +980,12 @@ public:
     //! many markers have been defined.
     int markerDefine(const QPixmap &pm, int markerNumber = -1);
 
+    //! Define a marker using a copy of the image \a im with the marker number
+    //! \a markerNumber.  If \a markerNumber is -1 then the marker number is
+    //! automatically allocated.  The marker number is returned or -1 if too
+    //! many markers have been defined.
+    int markerDefine(const QImage &im, int markerNumber = -1);
+
     //! Add an instance of marker number \a markerNumber to line number
     //! \a linenr.  A handle for the marker is returned which can be used to
     //! track the marker's position, or -1 if the \a markerNumber was invalid.
@@ -931,6 +1034,11 @@ public:
     //! \sa markerFindNext()
     int markerFindPrevious(int linenr, unsigned mask) const;
 
+    //! Returns true if text entered by the user will overwrite existing text.
+    //!
+    //! \sa setOverwriteMode()
+    bool overwriteMode() const;
+
     //! Returns the widget's paper (ie. background) colour.
     //!
     //! \sa setPaper()
@@ -962,10 +1070,16 @@ public:
     //! \sa clearRegisteredImages(), QsciLexer::apiLoad()
     void registerImage(int id, const QPixmap &pm);
 
-    //! Replace the current selection, set by a previous call to findFirst() or
-    //! findNext(), with \a replaceStr.
+    //! Register an image \a im with ID \a id.  Registered images can be
+    //! displayed in auto-completion lists.
     //!
-    //! \sa findFirst(), findNext()
+    //! \sa clearRegisteredImages(), QsciLexer::apiLoad()
+    void registerImage(int id, const QImage &im);
+
+    //! Replace the current selection, set by a previous call to findFirst(),
+    //! findFirstInSelection() or findNext(), with \a replaceStr.
+    //!
+    //! \sa findFirst(), findFirstInSelection(), findNext()
     virtual void replace(const QString &replaceStr);
 
     //! Reset the fold margin colours to their defaults.
@@ -1021,6 +1135,11 @@ public:
     //! dark blue.
     void setCallTipsHighlightColor(const QColor &col);
 
+    //! Set the current call tip position.  The default is CallTipsBelowText.
+    //!
+    //! \sa callTipsPosition()
+    void setCallTipsPosition(CallTipsPosition position);
+
     //! Set the current call tip style.  The default is CallTipsNoContext.
     //!
     //! \sa callTipsStyle()
@@ -1081,6 +1200,16 @@ public:
     //! If \a indicatorNumber is -1 then the colour of all indicators is set.
     void setIndicatorForegroundColor(const QColor &col, int indicatorNumber = -1);
 
+    //! Set the outline colour of indicator \a indicatorNumber to \a col.
+    //! If \a indicatorNumber is -1 then the colour of all indicators is set.
+    //! At the moment only the alpha value of the colour has any affect.
+    void setIndicatorOutlineColor(const QColor &col, int indicatorNumber = -1);
+
+    //! Set the margin options to \a options.
+    //!
+    //! \sa marginOptions(), MoNone, MoSublineSelect.
+    void setMarginOptions(int options);
+
     //! Set the margin text of line \a line with the text \a text using the
     //! style number \a style.
     void setMarginText(int line, const QString &text, int style);
@@ -1119,30 +1248,52 @@ public:
     //! \sa setMarkerBackgroundColor()
     void setMarkerForegroundColor(const QColor &col, int markerNumber = -1);
 
-    //! Set the background colour used to display matched braces to \a col.
-    //! The default is white.
+    //! Set the background colour used to display matched braces to \a col.  It
+    //! is ignored if an indicator is being used.  The default is white.
     //!
-    //! \sa setMatchedBraceForegroundColor()
+    //! \sa setMatchedBraceForegroundColor(), setMatchedBraceIndicator()
     void setMatchedBraceBackgroundColor(const QColor &col);
 
-    //! Set the foreground colour used to display matched braces to \a col.
-    //! The default is red.
+    //! Set the foreground colour used to display matched braces to \a col.  It
+    //! is ignored if an indicator is being used.  The default is red.
     //!
-    //! \sa setMatchedBraceBackgroundColor()
+    //! \sa setMatchedBraceBackgroundColor(), setMatchedBraceIndicator()
     void setMatchedBraceForegroundColor(const QColor &col);
 
-    //! Set the background colour used to display unmatched braces to \a col.
-    //! The default is white.
+    //! Set the indicator used to display matched braces to \a indicatorNumber.
+    //! The default is not to use an indicator.
     //!
-    //! \sa setUnmatchedBraceForegroundColor()
+    //! \sa resetMatchedBraceIndicator(), setMatchedBraceBackgroundColor()
+    void setMatchedBraceIndicator(int indicatorNumber);
+
+    //! Stop using an indicator to display matched braces.
+    //!
+    //! \sa setMatchedBraceIndicator()
+    void resetMatchedBraceIndicator();
+
+    //! Set the background colour used to display unmatched braces to \a col.
+    //! It is ignored if an indicator is being used.  The default is white.
+    //!
+    //! \sa setUnmatchedBraceForegroundColor(), setUnmatchedBraceIndicator()
     void setUnmatchedBraceBackgroundColor(const QColor &col);
 
     //! Set the foreground colour used to display unmatched braces to \a col.
-    //! The default is blue.
+    //! It is ignored if an indicator is being used.  The default is blue.
     //!
-    //! \sa setUnmatchedBraceBackgroundColor()
+    //! \sa setUnmatchedBraceBackgroundColor(), setUnmatchedBraceIndicator()
     void setUnmatchedBraceForegroundColor(const QColor &col);
 
+    //! Set the indicator used to display unmatched braces to
+    //! \a indicatorNumber.  The default is not to use an indicator.
+    //!
+    //! \sa resetUnmatchedBraceIndicator(), setUnmatchedBraceBackgroundColor()
+    void setUnmatchedBraceIndicator(int indicatorNumber);
+
+    //! Stop using an indicator to display unmatched braces.
+    //!
+    //! \sa setUnmatchedBraceIndicator()
+    void resetUnmatchedBraceIndicator();
+
     //! Set the visual flags displayed when a line is wrapped.  \a endFlag
     //! determines if and where the flag at the end of a line is displayed.
     //! \a startFlag determines if and where the flag at the start of a line is
@@ -1181,6 +1332,12 @@ public:
     //! \sa extraDescent(), setExtraAscent()
     void setExtraDescent(int extra);
 
+    //! Text entered by the user will overwrite existing text if \a overwrite
+    //! is true.
+    //!
+    //! \sa overwriteMode()
+    void setOverwriteMode(bool overwrite);
+
     //! Sets the background colour of visible whitespace to \a col.  If \a col
     //! is an invalid color (the default) then the color specified by the
     //! current lexer is used.
@@ -1249,6 +1406,10 @@ public:
     //! \sa setWhitespaceVisibility()
     WhitespaceVisibility whitespaceVisibility() const;
 
+    //! Returns the word at the \a line line number and \a index character
+    //! index.
+    QString wordAtLineIndex(int line, int index) const;
+
     //! Returns the word at the \a point pixel coordinates.
     QString wordAtPoint(const QPoint &point) const;
 
@@ -1729,8 +1890,8 @@ signals:
     //! This signal is emitted whenever the user clicks on an indicator.  \a
     //! line is the number of the line where the user clicked.  \a index is the
     //! character index within the line.  \a state is the state of the modifier
-    //! keys (Qt::ShiftModifier, Qt::ControlModifier and Qt::AltModifer) when
-    //! the user clicked.
+    //! keys (Qt::ShiftModifier, Qt::ControlModifier, Qt::AltModifer and
+    //! Qt::MetaModifier) when the user clicked.
     //!
     //! \sa indicatorReleased()
     void indicatorClicked(int line, int index, Qt::KeyboardModifiers state);
@@ -1738,8 +1899,8 @@ signals:
     //! This signal is emitted whenever the user releases the mouse on an
     //! indicator.  \a line is the number of the line where the user clicked.
     //! \a index is the character index within the line.  \a state is the state
-    //! of the modifier keys (Qt::ShiftModifier, Qt::ControlModifier and
-    //! Qt::AltModifer) when the user released the mouse.
+    //! of the modifier keys (Qt::ShiftModifier, Qt::ControlModifier,
+    //! Qt::AltModifer and Qt::MetaModifier) when the user released the mouse.
     //!
     //! \sa indicatorClicked()
     void indicatorReleased(int line, int index, Qt::KeyboardModifiers state);
@@ -1750,8 +1911,8 @@ signals:
     //! This signal is emitted whenever the user clicks on a sensitive margin.
     //! \a margin is the margin.  \a line is the number of the line where the
     //! user clicked.  \a state is the state of the modifier keys
-    //! (Qt::ShiftModifier, Qt::ControlModifier and Qt::AltModifer) when the
-    //! user clicked.
+    //! (Qt::ShiftModifier, Qt::ControlModifier, Qt::AltModifer and
+    //! Qt::MetaModifier) when the user clicked.
     //!
     //! \sa marginSensitivity(), setMarginSensitivity()
     void marginClicked(int margin, int line, Qt::KeyboardModifiers state);
@@ -1783,6 +1944,13 @@ signals:
     //! \sa showUserList()
     void userListActivated(int id, const QString &string);
 
+protected:
+    //! \reimp
+    virtual bool event(QEvent *e);
+
+    //! \reimp
+    virtual void contextMenuEvent(QContextMenuEvent *e);
+
 private slots:
     void handleCallTipClick(int dir);
     void handleCharAdded(int charadded);
@@ -1806,9 +1974,9 @@ private slots:
 
     void handleUpdateUI(int updated);
 
-private:
-    typedef QByteArray ScintillaString;
+    void delete_selection();
 
+private:
     void detachLexer();
 
     enum IndentState {
@@ -1838,8 +2006,6 @@ private:
     void foldExpand(int &line, bool doExpand, bool force = false,
             int visLevels = 0, int level = -1);
     void setFoldMarker(int marknr, int mark = SC_MARK_EMPTY);
-    QString convertTextS2Q(const char *s) const;
-    ScintillaString convertTextQ2S(const QString &q) const;
     void setLexerStyle(int style);
     void setStylesFont(const QFont &f, int style);
 
@@ -1861,20 +2027,29 @@ private:
     void insertAtPos(const QString &text, int pos);
     static int mapModifiers(int modifiers);
 
-    ScintillaString styleText(const QList<QsciStyledText> &styled_text,
+    QString wordAtPosition(int position) const;
+
+    ScintillaBytes styleText(const QList<QsciStyledText> &styled_text,
             char **styles, int style_offset = 0);
 
     struct FindState
     {
-        FindState() : inProgress(0) {}
+        enum Status
+        {
+            Finding,
+            FindingInSelection,
+            Idle
+        };
 
-        bool inProgress;
+        FindState() : status(Idle) {}
+
+        Status status;
         QString expr;
         bool wrap;
         bool forward;
         int flags;
-        long startpos;
-        long endpos;
+        long startpos, startpos_orig;
+        long endpos, endpos_orig;
         bool show;
     };
 
@@ -1893,6 +2068,7 @@ private:
     int acThresh;
     QStringList wseps;
     const char *wchars;
+    CallTipsPosition call_tips_position;
     CallTipsStyle call_tips_style;
     int maxCallTips;
     QStringList ct_entries;
@@ -1907,14 +2083,16 @@ private:
     QByteArray explicit_fillups;
     bool fillups_enabled;
 
-    // The following allow ListBoxQt to distinguish between an auto-completion
-    // list and a user list, and to return the full selection of an
-    // auto-completion list.
-    friend class ListBoxQt;
+    // The following allow QsciListBoxQt to distinguish between an
+    // auto-completion list and a user list, and to return the full selection
+    // of an auto-completion list.
+    friend class QsciListBoxQt;
 
     QString acSelection;
     bool isAutoCompletionList() const;
 
+    void set_shortcut(QAction *action, QsciCommand::Command cmd_id) const;
+
     QsciScintilla(const QsciScintilla &);
     QsciScintilla &operator=(const QsciScintilla &);
 };
