@@ -1,6 +1,6 @@
 // This class defines the "official" low-level API.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -38,6 +33,7 @@ extern "C++" {
 #include <qglobal.h>
 
 #include <QAbstractScrollArea>
+#include <QByteArray>
 
 #include <qpoint.h>
 #include <qtimer.h>
@@ -47,12 +43,13 @@ extern "C++" {
 
 QT_BEGIN_NAMESPACE
 class QColor;
+class QImage;
 class QPainter;
 class QPixmap;
 class QMimeData;
 QT_END_NAMESPACE
 
-class ScintillaQt;
+class QsciScintillaQt;
 
 
 //! \brief The QsciScintillaBase class implements the Scintilla editor widget
@@ -228,15 +225,13 @@ public:
         //!
         SCI_SETCODEPAGE = 2037,
 
-        //!
-        SCI_SETUSEPALETTE = 2039,
-
         //! This message sets the symbol used to draw one of 32 markers.  Some
         //! markers have pre-defined uses, see the SC_MARKNUM_* values.
         //! \a wParam is the number of the marker.
         //! \a lParam is the marker symbol and is one of the SC_MARK_* values.
         //!
-        //! \sa SCI_MARKERADD, SCI_MARKERDEFINEPIXMAP
+        //! \sa SCI_MARKERADD, SCI_MARKERDEFINEPIXMAP,
+        //! SCI_MARKERDEFINERGBAIMAGE
         SCI_MARKERDEFINE = 2040,
 
         //! This message sets the foreground colour used to draw a marker.  A
@@ -312,7 +307,7 @@ public:
         //! ports of Scintilla this is a pointer to either raw or textual XPM
         //! image data.
         //!
-        //! \sa SCI_MARKERDEFINE
+        //! \sa SCI_MARKERDEFINE, SCI_MARKERDEFINERGBAIMAGE
         SCI_MARKERDEFINEPIXMAP = 2049,
 
         //! This message sets what can be displayed in a margin.
@@ -419,6 +414,18 @@ public:
         //!
         SCI_STYLESETCASE = 2060,
 
+        //!
+        SCI_STYLESETSIZEFRACTIONAL = 2061,
+
+        //!
+        SCI_STYLEGETSIZEFRACTIONAL = 2062,
+
+        //!
+        SCI_STYLESETWEIGHT = 2063,
+
+        //!
+        SCI_STYLEGETWEIGHT = 2064,
+
         //!
         SCI_STYLESETCHARACTERSET = 2066,
 
@@ -629,9 +636,6 @@ public:
         //!
         SCI_GETCARETFORE = 2138,
 
-        //!
-        SCI_GETUSEPALETTE = 2139,
-
         //! This message returns a non-zero value if the document is read-only.
         //! 
         //! \sa SCI_SETREADONLY
@@ -865,6 +869,9 @@ public:
         //!
         SCI_CALLTIPUSESTYLE = 2212,
 
+        //!
+        SCI_CALLTIPSETPOSITION = 2213,
+
         //!
         SCI_VISIBLEFROMDOCLINE = 2220,
 
@@ -913,6 +920,18 @@ public:
         //!
         SCI_WRAPCOUNT = 2235,
 
+        //!
+        SCI_GETALLLINESVISIBLE = 2236,
+
+        //!
+        SCI_FOLDLINE = 2237,
+
+        //!
+        SCI_FOLDCHILDREN = 2238,
+
+        //!
+        SCI_EXPANDCHILDREN = 2239,
+
         //!
         SCI_SETTABINDENTS = 2260,
 
@@ -1007,6 +1026,12 @@ public:
         //!
         SCI_SETFOLDMARGINHICOLOUR = 2291,
 
+        //!
+        SCI_MARKERSETBACKSELECTED = 2292,
+
+        //!
+        SCI_MARKERENABLEHIGHLIGHT = 2293,
+
         //!
         SCI_LINEDOWN = 2300,
 
@@ -1317,7 +1342,7 @@ public:
         //! ports of Scintilla this is a pointer to either raw or textual XPM
         //! image data.
         //!
-        //! \sa SCI_CLEARREGISTEREDIMAGES
+        //! \sa SCI_CLEARREGISTEREDIMAGES, SCI_REGISTERRGBAIMAGE
         SCI_REGISTERIMAGE = 2405,
 
         //!
@@ -1328,7 +1353,7 @@ public:
 
         //! This message de-registers all currently registered images.
         //!
-        //! \sa SCI_REGISTERIMAGE
+        //! \sa SCI_REGISTERIMAGE, SCI_REGISTERRGBAIMAGE
         SCI_CLEARREGISTEREDIMAGES = 2408,
 
         //!
@@ -1583,6 +1608,12 @@ public:
         //!
         SCI_GETHOTSPOTSINGLELINE = 2497,
 
+        //!
+        SCI_BRACEHIGHLIGHTINDICATOR = 2498,
+
+        //!
+        SCI_BRACEBADLIGHTINDICATOR = 2499,
+
         //!
         SCI_SETINDICATORCURRENT = 2500,
 
@@ -1702,6 +1733,9 @@ public:
         //!
         SCI_MARGINGETSTYLEOFFSET = 2538,
 
+        //!
+        SCI_SETMARGINOPTIONS = 2539,
+
         //!
         SCI_ANNOTATIONSETTEXT = 2540,
 
@@ -1738,6 +1772,24 @@ public:
         //!
         SCI_ANNOTATIONGETSTYLEOFFSET = 2551,
 
+        //!
+        SCI_RELEASEALLEXTENDEDSTYLES = 2552,
+
+        //!
+        SCI_ALLOCATEEXTENDEDSTYLES = 2553,
+
+        //!
+        SCI_SETEMPTYSELECTION = 2556,
+
+        //!
+        SCI_GETMARGINOPTIONS = 2557,
+
+        //!
+        SCI_INDICSETOUTLINEALPHA = 2558,
+
+        //!
+        SCI_INDICGETOUTLINEALPHA = 2559,
+
         //!
         SCI_ADDUNDOACTION = 2560,
 
@@ -1765,6 +1817,9 @@ public:
         //!
         SCI_GETADDITIONALCARETSBLINK = 2568,
 
+        //!
+        SCI_SCROLLRANGE = 2569,
+
         //!
         SCI_SETADDITIONALCARETSVISIBLE = 2608,
 
@@ -1915,6 +1970,156 @@ public:
         //!
         SCI_VERTICALCENTRECARET = 2619,
 
+        //!
+        SCI_MOVESELECTEDLINESUP = 2620,
+
+        //!
+        SCI_MOVESELECTEDLINESDOWN = 2621,
+
+        //!
+        SCI_SETIDENTIFIER = 2622,
+
+        //!
+        SCI_GETIDENTIFIER = 2623,
+
+        //! This message sets the width of an RGBA image specified by a future
+        //! call to SCI_MARKERDEFINERGBAIMAGE or SCI_REGISTERRGBAIMAGE.
+        //!
+        //! \sa SCI_RGBAIMAGESETHEIGHT, SCI_MARKERDEFINERGBAIMAGE,
+        //! SCI_REGISTERRGBAIMAGE.
+        SCI_RGBAIMAGESETWIDTH = 2624,
+
+        //! This message sets the height of an RGBA image specified by a future
+        //! call to SCI_MARKERDEFINERGBAIMAGE or SCI_REGISTERRGBAIMAGE.
+        //!
+        //! \sa SCI_RGBAIMAGESETWIDTH, SCI_MARKERDEFINERGBAIMAGE,
+        //! SCI_REGISTERRGBAIMAGE.
+        SCI_RGBAIMAGESETHEIGHT = 2625,
+
+        //! This message sets the symbol used to draw one of the 32 markers to
+        //! an RGBA image.  RGBA images use the SC_MARK_RGBAIMAGE marker
+        //! symbol.
+        //! \a wParam is the number of the marker.
+        //! \a lParam is a pointer to a QImage instance.  Note that in other
+        //! ports of Scintilla this is a pointer to raw RGBA image data.
+        //!
+        //! \sa SCI_MARKERDEFINE, SCI_MARKERDEFINEPIXMAP
+        SCI_MARKERDEFINERGBAIMAGE = 2626,
+
+        //! This message takes a copy of an image and registers it so that it
+        //! can be refered to by a unique integer identifier.
+        //! \a wParam is the image's identifier.
+        //! \a lParam is a pointer to a QImage instance.  Note that in other
+        //! ports of Scintilla this is a pointer to raw RGBA image data.
+        //!
+        //! \sa SCI_CLEARREGISTEREDIMAGES, SCI_REGISTERIMAGE
+        SCI_REGISTERRGBAIMAGE = 2627,
+
+        //!
+        SCI_SCROLLTOSTART = 2628,
+
+        //!
+        SCI_SCROLLTOEND = 2629,
+
+        //!
+        SCI_SETTECHNOLOGY = 2630,
+
+        //!
+        SCI_GETTECHNOLOGY = 2631,
+
+        //!
+        SCI_CREATELOADER = 2632,
+
+        //!
+        SCI_COUNTCHARACTERS = 2633,
+
+        //!
+        SCI_AUTOCSETCASEINSENSITIVEBEHAVIOUR = 2634,
+
+        //!
+        SCI_AUTOCGETCASEINSENSITIVEBEHAVIOUR = 2635,
+
+        //!
+        SCI_FINDINDICATORSHOW = 2640,
+
+        //!
+        SCI_FINDINDICATORFLASH = 2641,
+
+        //!
+        SCI_FINDINDICATORHIDE = 2642,
+
+        //!
+        SCI_GETRANGEPOINTER = 2643,
+
+        //!
+        SCI_GETGAPPOSITION = 2644,
+
+        //!
+        SCI_DELETERANGE = 2645,
+
+        //!
+        SCI_GETWORDCHARS = 2646,
+
+        //!
+        SCI_GETWHITESPACECHARS = 2647,
+
+        //!
+        SCI_SETPUNCTUATIONCHARS = 2648,
+
+        //!
+        SCI_GETPUNCTUATIONCHARS = 2649,
+
+        //!
+        SCI_GETSELECTIONEMPTY = 2650,
+
+        //!
+        SCI_RGBAIMAGESETSCALE = 2651,
+
+        //!
+        SCI_VCHOMEDISPLAY = 2652,
+
+        //!
+        SCI_GETCARETLINEVISIBLEALWAYS = 2654,
+
+        //!
+        SCI_SETCARETLINEVISIBLEALWAYS = 2655,
+
+        //!
+        SCI_VCHOMEDISPLAYEXTEND = 2653,
+
+        //!
+        SCI_AUTOCSETORDER = 2660,
+
+        //!
+        SCI_AUTOCGETORDER = 2661,
+
+        //!
+        SCI_FOLDALL = 2662,
+
+        //!
+        SCI_SETAUTOMATICFOLD = 2663,
+
+        //!
+        SCI_GETAUTOMATICFOLD = 2664,
+
+        //!
+        SCI_SETREPRESENTATION = 2665,
+
+        //!
+        SCI_GETREPRESENTATION = 2666,
+
+        //!
+        SCI_CLEARREPRESENTATION = 2667,
+
+        //!
+        SCI_SETMOUSESELECTIONRECTANGULARSWITCH = 2668,
+
+        //!
+        SCI_GETMOUSESELECTIONRECTANGULARSWITCH = 2669,
+
+        //!
+        SCI_POSITIONRELATIVE = 2670,
+
         //!
         SCI_STARTRECORD = 3001,
 
@@ -1976,7 +2181,7 @@ public:
         SCI_DESCRIBEPROPERTY = 4016,
 
         //!
-        SCI_DESCRIBEKEYWORDSETS = 4017
+        SCI_DESCRIBEKEYWORDSETS = 4017,
     };
 
     enum
@@ -2002,6 +2207,12 @@ public:
         SC_EFF_QUALITY_LCD_OPTIMIZED = 3
     };
 
+    enum
+    {
+        SC_MARGINOPTION_NONE = 0x00,
+        SC_MARGINOPTION_SUBLINESELECT = 0x01
+    };
+
     enum
     {
         SC_MULTIPASTE_ONCE = 0,
@@ -2027,7 +2238,8 @@ public:
     {
         SC_WRAPVISUALFLAG_NONE = 0x0000,
         SC_WRAPVISUALFLAG_END = 0x0001,
-        SC_WRAPVISUALFLAG_START = 0x0002
+        SC_WRAPVISUALFLAG_START = 0x0002,
+        SC_WRAPVISUALFLAG_MARGIN = 0x0004
     };
 
     enum
@@ -2177,6 +2389,9 @@ public:
         //! The line is underlined using the marker's background color.
         SC_MARK_UNDERLINE = 29,
 
+        //! A RGBA format image.
+        SC_MARK_RGBAIMAGE = 30,
+
         //! Characters can be used as symbols by adding this to the ASCII value
         //! of the character.
         SC_MARK_CHARACTER = 10000
@@ -2291,7 +2506,6 @@ public:
 
     enum
     {
-        INDIC_MAX = 31,
         INDIC_PLAIN = 0,
         INDIC_SQUIGGLE = 1,
         INDIC_TT = 2,
@@ -2300,7 +2514,16 @@ public:
         INDIC_HIDDEN = 5,
         INDIC_BOX = 6,
         INDIC_ROUNDBOX = 7,
+        INDIC_STRAIGHTBOX = 8,
+        INDIC_DASH = 9,
+        INDIC_DOTS = 10,
+        INDIC_SQUIGGLELOW = 11,
+        INDIC_DOTBOX = 12,
+        INDIC_SQUIGGLEPIXMAP = 13,
+        INDIC_COMPOSITIONTHICK = 14,
+
         INDIC_CONTAINER = 8,
+        INDIC_MAX = 31,
         INDIC0_MASK = 0x20,
         INDIC1_MASK = 0x40,
         INDIC2_MASK = 0x80,
@@ -2475,14 +2698,19 @@ public:
         //! Shift key.
         SCMOD_SHIFT = 1,
 
-        //! Control key.
+        //! Control key (the Command key on OS/X, the Ctrl key on other
+        //! platforms).
         SCMOD_CTRL = 2,
 
         //! Alt key.
         SCMOD_ALT = 4,
 
-        //! Meta key.
-        SCMOD_SUPER = 8
+        //! This is the same as SCMOD_META on all platforms.
+        SCMOD_SUPER = 8,
+
+        //! Meta key (the Ctrl key on OS/X, the Windows key on other
+        //! platforms).
+        SCMOD_META = 16
     };
 
     //! This enum defines the different language lexers.
@@ -2793,7 +3021,82 @@ public:
         SCLEX_A68K = 100,
 
         //! Select the Modula 3 lexer.
-        SCLEX_MODULA = 101
+        SCLEX_MODULA = 101,
+
+        //! Select the CoffeeScript lexer.
+        SCLEX_COFFEESCRIPT = 102,
+
+        //! Select the Take Command lexer.
+        SCLEX_TCMD = 103,
+
+        //! Select the AviSynth lexer.
+        SCLEX_AVS = 104,
+
+        //! Select the ECL lexer.
+        SCLEX_ECL = 105,
+
+        //! Select the OScript lexer.
+        SCLEX_OSCRIPT = 106,
+
+        //! Select the Visual Prolog lexer.
+        SCLEX_VISUALPROLOG = 107,
+
+        //! Select the Literal Haskell lexer.
+        SCLEX_LITERATEHASKELL = 108,
+
+        //! Select the Structured Text lexer.
+        SCLEX_STTXT = 109,
+
+        //! Select the KVIrc lexer.
+        SCLEX_KVIRC = 110,
+
+        //! Select the Rust lexer.
+        SCLEX_RUST = 111,
+    };
+
+    enum
+    {
+        SC_WEIGHT_NORMAL = 400,
+        SC_WEIGHT_SEMIBOLD = 600,
+        SC_WEIGHT_BOLD = 700,
+    };
+
+    enum
+    {
+        SC_TECHNOLOGY_DEFAULT = 0,
+        SC_TECHNOLOGY_DIRECTWRITE = 1,
+    };
+
+    enum
+    {
+        SC_CASEINSENSITIVEBEHAVIOUR_RESPECTCASE = 0,
+        SC_CASEINSENSITIVEBEHAVIOUR_IGNORECASE = 1,
+    };
+
+    enum
+    {
+        SC_FONT_SIZE_MULTIPLIER = 100,
+    };
+
+    enum
+    {
+        SC_FOLDACTION_CONTRACT = 0,
+        SC_FOLDACTION_EXPAND = 1,
+        SC_FOLDACTION_TOGGLE = 2,
+    };
+
+    enum
+    {
+        SC_AUTOMATICFOLD_SHOW = 0x0001,
+        SC_AUTOMATICFOLD_CLICK = 0x0002,
+        SC_AUTOMATICFOLD_CHANGE = 0x0004,
+    };
+
+    enum
+    {
+        SC_ORDER_PRESORTED = 0,
+        SC_ORDER_PERFORMSORT = 1,
+        SC_ORDER_CUSTOM = 2,
     };
 
     //! Construct an empty QsciScintillaBase with parent \a parent.
@@ -2852,6 +3155,10 @@ public:
     long SendScintilla(unsigned int msg, unsigned long wParam,
             const QPixmap &lParam) const;
 
+    //! \overload
+    long SendScintilla(unsigned int msg, unsigned long wParam,
+            const QImage &lParam) const;
+
     //! Send the Scintilla message \a msg and return a pointer result.
     void *SendScintillaPtrResult(unsigned int msg) const;
 
@@ -2908,6 +3215,12 @@ signals:
     //!
     void SCN_DWELLSTART(int, int, int);
 
+    //! This signal is emitted when focus is received.
+    void SCN_FOCUSIN();
+
+    //! This signal is emitted when focus is lost.
+    void SCN_FOCUSOUT();
+
     //! This signal is emitted when the user clicks on text in a style with the
     //! hotspot attribute set.
     //! \a position is the position in the text where the click occured.
@@ -3061,7 +3374,8 @@ protected:
     virtual void keyPressEvent(QKeyEvent *e);
 
     //! Re-implemented to handle composed characters.
-    virtual void inputMethodEvent(QInputMethodEvent *e);
+    virtual void inputMethodEvent(QInputMethodEvent *event);
+    virtual QVariant inputMethodQuery(Qt::InputMethodQuery query) const;
 
     //! Re-implemented to handle mouse double-clicks.
     virtual void mouseDoubleClickEvent(QMouseEvent *e);
@@ -3084,6 +3398,21 @@ protected:
     //! \internal Re-implemented to handle scrolling.
     virtual void scrollContentsBy(int dx, int dy);
 
+    //! \internal This helps to work around some Scintilla bugs.
+    void setScrollBars();
+
+    //! \internal Qt4, Qt5 portability.
+    typedef QByteArray ScintillaBytes;
+
+#define ScintillaBytesConstData(b)  (b).constData()
+
+
+    //! \internal Convert a QString to encoded bytes.
+    ScintillaBytes textAsBytes(const QString &text) const;
+
+    //! \internal Convert encoded bytes to a QString.
+    QString bytesAsText(const char *bytes) const;
+
 private slots:
     void handleTimer();
     void handleVSb(int value);
@@ -3091,12 +3420,15 @@ private slots:
     void handleSelection();
 
 private:
-    // This is needed to allow ScintillaQt to emit this class's signals.
-    friend class ScintillaQt;
+    // This is needed to allow QsciScintillaQt to emit this class's signals.
+    friend class QsciScintillaQt;
 
-    ScintillaQt *sci;
+    QsciScintillaQt *sci;
     QPoint triple_click_at;
     QTimer triple_click;
+    int preeditPos;
+    int preeditNrBytes;
+    QString preeditString;
 
     void acceptAction(QDropEvent *e);
 
